"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var Promise = require("bluebird");
var form = require("resin-cli-form");
var visuals = require("resin-cli-visuals");
var resin = require("resin-sdk-preconfigured");
var chalk_1 = require("chalk");
var validation = require("./validation");
var messages = require("./messages");
function authenticate(options) {
    return form
        .run([
        {
            message: 'Email:',
            name: 'email',
            type: 'input',
            validate: validation.validateEmail,
        },
        {
            message: 'Password:',
            name: 'password',
            type: 'password',
        },
    ], { override: options })
        .then(resin.auth.login)
        .then(resin.auth.twoFactor.isPassed)
        .then(function (isTwoFactorAuthPassed) {
        if (isTwoFactorAuthPassed) {
            return;
        }
        return form
            .ask({
            message: 'Two factor auth challenge:',
            name: 'code',
            type: 'input',
        })
            .then(resin.auth.twoFactor.challenge)
            .catch(function (error) {
            return resin.auth.logout().then(function () {
                if (error.name === 'ResinRequestError' &&
                    error.statusCode === 401) {
                    throw new Error('Invalid two factor authentication code');
                }
                throw error;
            });
        });
    });
}
exports.authenticate = authenticate;
function askLoginType() {
    return form.ask({
        message: 'How would you like to login?',
        name: 'loginType',
        type: 'list',
        choices: [
            {
                name: 'Web authorization (recommended)',
                value: 'web',
            },
            {
                name: 'Credentials',
                value: 'credentials',
            },
            {
                name: 'Authentication token',
                value: 'token',
            },
            {
                name: "I don't have a Resin account!",
                value: 'register',
            },
        ],
    });
}
exports.askLoginType = askLoginType;
function selectDeviceType() {
    return resin.models.device.getSupportedDeviceTypes().then(function (deviceTypes) {
        return form.ask({
            message: 'Device Type',
            type: 'list',
            choices: deviceTypes,
        });
    });
}
exports.selectDeviceType = selectDeviceType;
function confirm(yesOption, message, yesMessage) {
    return Promise.try(function () {
        if (yesOption) {
            if (yesMessage) {
                console.log(yesMessage);
            }
            return true;
        }
        return form.ask({
            message: message,
            type: 'confirm',
            default: false,
        });
    }).then(function (confirmed) {
        if (!confirmed) {
            throw new Error('Aborted');
        }
    });
}
exports.confirm = confirm;
function selectApplication(filter) {
    return resin.models.application
        .hasAny()
        .then(function (hasAnyApplications) {
        if (!hasAnyApplications) {
            throw new Error("You don't have any applications");
        }
        return resin.models.application.getAll();
    })
        .filter(filter || _.constant(true))
        .then(function (applications) {
        return form.ask({
            message: 'Select an application',
            type: 'list',
            choices: _.map(applications, function (application) { return ({
                name: application.app_name + " (" + application.device_type + ")",
                value: application.app_name,
            }); }),
        });
    });
}
exports.selectApplication = selectApplication;
function selectOrCreateApplication() {
    return resin.models.application
        .hasAny()
        .then(function (hasAnyApplications) {
        if (!hasAnyApplications)
            return;
        return resin.models.application.getAll().then(function (applications) {
            var appOptions = _.map(applications, function (application) { return ({
                name: application.app_name + " (" + application.device_type + ")",
                value: application.app_name,
            }); });
            appOptions.unshift({
                name: 'Create a new application',
                value: null,
            });
            return form.ask({
                message: 'Select an application',
                type: 'list',
                choices: appOptions,
            });
        });
    })
        .then(function (application) {
        if (application) {
            return application;
        }
        return form.ask({
            message: 'Choose a Name for your new application',
            type: 'input',
            validate: validation.validateApplicationName,
        });
    });
}
exports.selectOrCreateApplication = selectOrCreateApplication;
function awaitDevice(uuid) {
    return resin.models.device.getName(uuid).then(function (deviceName) {
        var spinner = new visuals.Spinner("Waiting for " + deviceName + " to come online");
        var poll = function () {
            return resin.models.device.isOnline(uuid).then(function (isOnline) {
                if (isOnline) {
                    spinner.stop();
                    console.info("The device **" + deviceName + "** is online!");
                    return;
                }
                else {
                    spinner.start();
                    return Promise.delay(3000).then(poll);
                }
            });
        };
        console.info("Waiting for " + deviceName + " to connect to resin...");
        return poll().return(uuid);
    });
}
exports.awaitDevice = awaitDevice;
function inferOrSelectDevice(preferredUuid) {
    return resin.models.device
        .getAll()
        .filter(function (device) { return device.is_online; })
        .then(function (onlineDevices) {
        if (_.isEmpty(onlineDevices)) {
            throw new Error("You don't have any devices online");
        }
        var defaultUuid = _.map(onlineDevices, 'uuid').includes(preferredUuid)
            ? preferredUuid
            : onlineDevices[0].uuid;
        return form.ask({
            message: 'Select a device',
            type: 'list',
            default: defaultUuid,
            choices: _.map(onlineDevices, function (device) { return ({
                name: (device.name || 'Untitled') + " (" + device.uuid.slice(0, 7) + ")",
                value: device.uuid,
            }); }),
        });
    });
}
exports.inferOrSelectDevice = inferOrSelectDevice;
function selectFromList(message, choices) {
    return form.ask({
        message: message,
        type: 'list',
        choices: _.map(choices, function (s) { return ({
            name: s.name,
            value: s,
        }); }),
    });
}
exports.selectFromList = selectFromList;
function printErrorMessage(message) {
    console.error(chalk_1.default.red(message));
    console.error(chalk_1.default.red("\n" + messages.getHelp + "\n"));
}
exports.printErrorMessage = printErrorMessage;
function exitWithExpectedError(message) {
    if (message instanceof Error) {
        (_a = message, message = _a.message);
    }
    printErrorMessage(message);
    process.exit(1);
    var _a;
}
exports.exitWithExpectedError = exitWithExpectedError;
//# sourceMappingURL=patterns.js.map